@@ -42,13 +42,20 @@
 #include "tio/log.h"
 #include "tio/error.h"
 
+#ifdef HAVE_TERMIOS2
+extern int setspeed2(int fd, int baudrate);
+#endif
+
 static struct termios tio, tio_old, stdout_new, stdout_old, stdin_new, stdin_old;
 static unsigned long rx_total = 0, tx_total = 0;
 static bool connected = false;
 static bool tainted = false;
-static int fd;
 static bool print_mode = NORMAL;
+static bool standard_baudrate = true;
 static void (*print)(char c);
+static int fd;
+static bool map_inlcrnl = false;
+static bool map_odelbs = false;
 
 #define tio_printf(format, args...) \
 { \
@@ -276,24 +283,32 @@ void tty_configure(void)
         AUTOCONF_BAUDRATE_CASES
 
         default:
+#ifdef HAVE_TERMIOS2
+            standard_baudrate = false;
+            break;
+#else
             error_printf("Invalid baud rate");
             exit(EXIT_FAILURE);
+#endif
     }
 
-    // Set input speed
-    status = cfsetispeed(&tio, baudrate);
-    if (status == -1)
+    if (standard_baudrate)
     {
-        error_printf("Could not configure input speed (%s)", strerror(errno));
-        exit(EXIT_FAILURE);
-    }
+        // Set input speed
+        status = cfsetispeed(&tio, baudrate);
+        if (status == -1)
+        {
+            error_printf("Could not configure input speed (%s)", strerror(errno));
+            exit(EXIT_FAILURE);
+        }
 
-    // Set output speed
-    cfsetospeed(&tio, baudrate);
-    if (status == -1)
-    {
-        error_printf("Could not configure output speed (%s)", strerror(errno));
-        exit(EXIT_FAILURE);
+        // Set output speed
+        cfsetospeed(&tio, baudrate);
+        if (status == -1)
+        {
+            error_printf("Could not configure output speed (%s)", strerror(errno));
+            exit(EXIT_FAILURE);
+        }
     }
 
     /* Set databits */
@@ -382,7 +397,6 @@ void tty_configure(void)
     tio.c_cc[VMIN]  = 1; // Blocking read until 1 character received
 
     /* Configure any specified input or output mappings */
-
     buffer = strdup(option.map);
     while (token_found == true)
     {
@@ -399,10 +413,14 @@ void tty_configure(void)
                 tio.c_iflag |= IGNCR;
             else if (strcmp(token,"ICRNL") == 0)
                 tio.c_iflag |= ICRNL;
-            else if (strcmp(token,"ONLCR") == 0)
+            else if (strcmp(token,"ONLCRNL") == 0)
                 tio.c_oflag |= ONLCR;
             else if (strcmp(token,"OCRNL") == 0)
                 tio.c_oflag |= OCRNL;
+            else if (strcmp(token,"ODELBS") == 0)
+                map_odelbs = true;
+            else if (strcmp(token,"INLCRNL") == 0)
+                map_inlcrnl = true;
             else
             {
                 printf("Error: Unknown mapping flag %s\n", token);
@@ -551,6 +569,17 @@ int tty_connect(void)
         goto error_tcsetattr;
     }
 
+#ifdef HAVE_TERMIOS2
+    if (!standard_baudrate)
+    {
+        if (setspeed2(fd, option.baudrate) != 0)
+        {
+            error_printf_silent("Could not set baudrate speed (%s)", strerror(errno));
+            goto error_setspeed2;
+        }
+    }
+#endif
+
     maxfd = MAX(fd, STDIN_FILENO) + 1;  /* Maximum bit entry (fd) to test */
 
     /* Input loop */
@@ -572,8 +601,16 @@ int tty_connect(void)
                     /* Update receive statistics */
                     rx_total++;
 
-                    /* Print received tty character to stdout */
-                    print(input_char);
+                    /* Map input character */
+                    if ((input_char == '\n') && (map_inlcrnl))
+                    {
+                        print('\r');
+                        print('\n');
+                    } else
+                    {
+                        /* Print received tty character to stdout */
+                        print(input_char);
+                    }
                     fflush(stdout);
 
                     /* Write to log */
@@ -611,6 +648,10 @@ int tty_connect(void)
 
                 if (forward)
                 {
+                    /* Map output character */
+                    if ((output_char == 127) && (map_odelbs))
+                        output_char = '\b';
+
                     /* Send output to tty device */
                     status = write(fd, &output_char, 1);
                     if (status < 0)
@@ -636,6 +677,9 @@ int tty_connect(void)
 
     return TIO_SUCCESS;
 
+#ifdef HAVE_TERMIOS2
+error_setspeed2:
+#endif
 error_tcsetattr:
 error_tcgetattr:
 error_read:
